#include "game.h"
#include "walls.h"
#include "player.h"
#include "../common/sources/system/fileCache.h"
#include "../common/sources/system/cgraphics2d.h"
#include <QDebug>
#include <QPainter>
#include "interface.h"
#include "bridge.h"
#include "mouse.h"

CGame   game;

//-----------------------------------------------------------------------------------------
CGame::CGame()
{
    characters[0].picture = 4;
    characters[0].name = "ZED";
    characters[0].stats[CCharacter::eStatHealth].value = 10;
    characters[0].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[0].stats[CCharacter::eStatStamina].value = 10;
    characters[0].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[0].stats[CCharacter::eStatMana].value = 10;
    characters[0].stats[CCharacter::eStatMana].maxValue = 10;
    characters[0].bodyObjects[CCharacter::eBodyPartRightHand] = -1;

    characters[1].picture = 16;
    characters[1].name = "STAMM";
    characters[1].stats[CCharacter::eStatHealth].value = 5;
    characters[1].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[1].stats[CCharacter::eStatStamina].value = 5;
    characters[1].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[1].stats[CCharacter::eStatMana].value = 5;
    characters[1].stats[CCharacter::eStatMana].maxValue = 10;
    characters[1].bodyObjects[CCharacter::eBodyPartRightHand] = -1;

    characters[2].picture = 22;
    characters[2].name = "WUUF";
    characters[2].stats[CCharacter::eStatHealth].value = 0;
    characters[2].stats[CCharacter::eStatHealth].maxValue = 10;
    characters[2].stats[CCharacter::eStatStamina].value = 10;
    characters[2].stats[CCharacter::eStatStamina].maxValue = 10;
    characters[2].stats[CCharacter::eStatMana].value = 10;
    characters[2].stats[CCharacter::eStatMana].maxValue = 10;
    characters[2].bodyObjects[CCharacter::eBodyPartRightHand] = -1;
}

//-----------------------------------------------------------------------------------------
void CGame::drawWall(QImage* image, CVec2 mapPos, CVec2 tablePos, EWallSide side, bool flip)
{
    CTile*  tile = map.getTile(mapPos);

    if (tile != NULL)
    {
        EWallSide playerSide = player.getWallSide(side);

        if (tile->mWalls[playerSide].getType() != eWallNothing)
        {
            const char* fileName = walls.getFilename(tablePos, side, flip);

            if (fileName[0] != 0)
            {
                // draw the wall
                QImage  wallGfx = fileCache.getImage(fileName);
                CVec2 gfxPos = walls.getGfxPos(tablePos, side, flip);

                graph2D.drawImage(image, gfxPos, wallGfx, 0, flip);

                // draw the ornates
                CWall*  wall = &tile->mWalls[playerSide];

                if (wall->getType() == eWallSimple)
                {
                    int ornate = ((CParamOrnate*)wall->mParams[0])->mValue;
                    walls.drawOrnate(image, tablePos, side, ornate);
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------------------
void CGame::displayMainView(QImage* image)
{
    if (interface.isInInventory() == false)
    {
        // draw the floor and the ceiling
        QImage  floorGfx = fileCache.getImage("gfx/3DView/Floor.png");
        QImage  ceilingGfx = fileCache.getImage("gfx/3DView/Ceiling.png");

        bool flip = (player.pos.x + player.pos.y + player.dir) & 1;

        graph2D.drawImage(image, CVec2(0, 33), ceilingGfx, 0, flip);
        graph2D.drawImage(image, CVec2(0, 72), floorGfx, 0, flip);

        // draw the walls
        CVec2   tablePos;
        CVec2   localPos;
        CVec2   mapPos;

        for (int y = 0; y < WALL_TABLE_HEIGHT; ++y)
            for (int x = 0; x < WALL_TABLE_WIDTH; ++x)
            {
                tablePos = walls.getDrawSequence(CVec2(x, y));
                localPos = walls.getMapPos(tablePos);
                mapPos = player.getPosFromLocal(localPos);

                drawWall(image, mapPos, tablePos, eWallSideUp, flip);
                drawWall(image, mapPos, tablePos, eWallSideLeft, flip);
                drawWall(image, mapPos, tablePos, eWallSideRight, flip);
            }
    }
}

//-----------------------------------------------------------------------------------------
QImage CGame::mainLoop()
{
    QImage image(SCREEN_WIDTH,
                 SCREEN_HEIGHT,
                 QImage::Format_RGBA8888);

    mouse.clearAreas();

    // clear the screen
    image.fill(QColor(0, 0, 0));

    // display
    displayMainView(&image);
    interface.display(&image);

    // processing
    mouse.update();
    SMouseArea* clickedArea = mouse.clickedArea();

    interface.update(clickedArea);

    return image;
}
