#include "interface.h"
#include "game.h"
#include "../common/sources/system/cgraphics2d.h"
#include "../common/sources/system/filecache.h"
#include <QDebug>

CInterface   interface;

static const int   championsColors[4][3] =
{
    {  0, 222,   0},
    {255, 255,   0},
    {255,   0,   0},
    {  0,   0, 255}
};

//------------------------------------------------------------
CInterface::CInterface()
{
    mainState = eMainGame;
    weaponsAreaState = eWeaponsAreaWeapons;
    currentChampion = -1;
    currentWeapon = -1;

    for (int i = 0; i < 4; ++i)
    {
        weaponCoolDown[i] = new QTimer();
        weaponCoolDown[i]->setSingleShot(true);
    }
}

//------------------------------------------------------------
CInterface::~CInterface()
{
    for (int i = 0; i < 4; ++i)
        delete weaponCoolDown[i];
}

//------------------------------------------------------------
void    CInterface::display(QImage* image)
{
    if (isInInventory() == true)
        drawInventory(image);

    for (int i = 0; i < 4; ++i)
        drawChampion(image, i);

    drawSpells(image);
    drawWeaponsArea(image);
    drawArrows(image);
}

//------------------------------------------------------------
bool    CInterface::isInInventory()
{
    return (mainState == eMainInventory);
}

//------------------------------------------------------------
bool    CInterface::isChampionEmpty(int num)
{
    CCharacter* c = &game.characters[num];
    return c->picture == -1;
}

//------------------------------------------------------------
bool    CInterface::isChampionDead(int num)
{
    CCharacter* c = &game.characters[num];
    return c->isDead();
}

//------------------------------------------------------------
bool    CInterface::isWeaponEmpty(int num)
{
    return (isChampionDead(num) || isChampionEmpty(num));
}

//------------------------------------------------------------
bool    CInterface::isWeaponGreyed(int num)
{
    return weaponCoolDown[num]->isActive();
}

//------------------------------------------------------------
bool    CInterface::isArrowsGreyed()
{
    return isInInventory();
}

//------------------------------------------------------------
void    CInterface::drawBar(QImage* image, int num, CVec2 pos, CCharacter::EStatsNames statName)
{
    CCharacter::SStat   stat = getStat(num, statName);

    pos.y = 27 - (25 * stat.value) / stat.maxValue;
    CRect   rect(pos, CVec2(pos.x + 3, 26));
    QColor color(championsColors[num][0],
                 championsColors[num][1],
                 championsColors[num][2]);
    graph2D.rect(image, rect, color, true);
}

//------------------------------------------------------------
void    CInterface::drawChampion(QImage* image, int num)
{
    if (isChampionEmpty(num) == false)
    {
        CRect   barsMouseRect(CVec2(69 * num + 43, 0), CVec2(69 * num + 66, 28));

        if (isChampionDead(num) == true)
        {
            QImage  champBg = fileCache.getImage("gfx/interface/DeadChampInfo.png");
            CVec2   pos(69 * num, 0);
            graph2D.drawImage(image, pos, champBg);

            // draw name
            CCharacter* c = &game.characters[num];
            CVec2   pos3(69 * num + 1, 1);
            drawText(image, pos3, eFontStandard, c->name.c_str(), QColor(173, 173, 173));
        }
        else
        {
            QImage  champBg = fileCache.getImage("gfx/interface/ChampInfo.png");
            CVec2   pos(69 * num, 0);
            graph2D.drawImage(image, pos, champBg);

            if (isInInventory() == true && currentChampion == num)
            {
                // draw a background rectangle
                CVec2   pos2(69 * num, 0);
                CRect   rect2(pos2, CVec2(pos2.x + 42, 6));
                graph2D.rect(image, rect2, QColor(68, 68, 68), true);

                CVec2   pos(69 * num + 7, 0);
                CRect   rect(pos, CVec2(pos.x + CHAMP_PORTRAIT_WIDTH - 1, pos.y + CHAMP_PORTRAIT_HEIGHT - 1));
                graph2D.rect(image, rect, QColor(102, 102, 102), true);

                // draw head
                CCharacter* c = &game.characters[num];
                rect = getChampionPortraitRect(c->picture);
                QImage  portraits = fileCache.getImage("gfx/interface/ChampPortraits.png");
                graph2D.drawImageAtlas(image, pos, portraits, rect);

                // mouse area
                mouse.addArea(eMouseArea_3DView, barsMouseRect);
            }
            else
            {
                // draw rectangles
                CRect   rect(CVec2(69 * num + 3, 9),
                             CVec2(69 * num + 20, 26));
                graph2D.rect(image, rect, QColor(102, 102, 102), false);

                CRect   rect2(CVec2(69 * num + 23, 9),
                             CVec2(69 * num + 40, 26));
                graph2D.rect(image, rect2, QColor(102, 102, 102), false);

                // draw hands
                QImage  bodyParts = fileCache.getImage("gfx/interface/Items6.png");

                rect = getBodyPartRect(CCharacter::eBodyPartLeftHand);
                CVec2   pos(69 * num + 4, 10);
                graph2D.drawImageAtlas(image, pos, bodyParts, rect);

                rect = getBodyPartRect(CCharacter::eBodyPartRightHand);
                CVec2   pos2(69 * num + 24, 10);
                graph2D.drawImageAtlas(image, pos2, bodyParts, rect);

                // draw name
                CCharacter* c = &game.characters[num];
                CVec2   pos3(69 * num + 1, 1);
                drawText(image, pos3, eFontStandard, c->name.c_str(), QColor(173, 173, 173));

                // mouse area
                mouse.addArea(eMouseArea_Inventory, barsMouseRect, (void*)num);
            }

            // bars
            drawBar(image, num, CVec2(69 * num + 46, 2), CCharacter::eStatHealth);
            drawBar(image, num, CVec2(69 * num + 53, 2), CCharacter::eStatStamina);
            drawBar(image, num, CVec2(69 * num + 60, 2), CCharacter::eStatMana);
        }
    }
}

//------------------------------------------------------------
void    CInterface::drawSpells(QImage* image)
{
    QImage  spellsBg = fileCache.getImage("gfx/interface/SpellsArea.png");
    CVec2   pos(233, 50);
    graph2D.drawImage(image, pos, spellsBg);

    for (int i = 0; i < 6; ++i)
    {
        CVec2 pos(239 + i * 14, 54);
        char word[2] = "a";
        word[0] += i;
        drawText(image, pos, eFontStandard, word, MAIN_INTERFACE_COLOR);
    }
}

//------------------------------------------------------------
void    CInterface::drawWeaponsArea(QImage* image)
{
    if (weaponsAreaState == eWeaponsAreaWeapons)
    {
        for (int i = 0; i < 4; ++i)
            drawWeapon(image, i);
    }
    else if (weaponsAreaState == eWeaponsAreaAttacks)
    {
        QImage  spellsBg = fileCache.getImage("gfx/interface/WeaponsActions.png");
        CVec2   pos(233, 77);
        graph2D.drawImage(image, pos, spellsBg);
    }
    else
    {
        QImage  spellsBg = fileCache.getImage("gfx/interface/DamageDone.png");
        CVec2   pos(232, 77);
        graph2D.drawImage(image, pos, spellsBg);
    }
}

//------------------------------------------------------------
void    CInterface::greyRectangle(QImage* image, CRect rect)
{
    for (int y = rect.tl.y; y <= rect.br.y; ++y)
        for (int x = rect.tl.x; x <= rect.br.x; ++x)
            if (((x + y) & 1) == 0)
                image->setPixel(QPoint(x, y), 0);
}

//------------------------------------------------------------
void    CInterface::drawWeapon(QImage* image, int num)
{
    if (isWeaponEmpty(num) == false)
    {
        CRect   rect(CVec2(233 + 22 * num, 86),
                     CVec2(252 + 22 * num, 120));
        graph2D.rect(image, rect, MAIN_INTERFACE_COLOR, true);

        if (isWeaponGreyed(num) == true)
            greyRectangle(image, rect);
    }
}

//------------------------------------------------------------
void    CInterface::drawArrows(QImage* image)
{
    QImage  arrowsBg = fileCache.getImage("gfx/interface/MovementArrows.png");
    CVec2   pos(233, 124);
    graph2D.drawImage(image, pos, arrowsBg);

    if (isArrowsGreyed() == true)
    {
        CRect   rect(CVec2(233, 124),
                     CVec2(319, 168));
        greyRectangle(image, rect);
    }
}

//------------------------------------------------------------
void    CInterface::drawInventory(QImage* image)
{
    QImage  inventoryBg = fileCache.getImage("gfx/interface/InventoryMain.png");
    CVec2   pos(0, 33);
    graph2D.drawImage(image, pos, inventoryBg);

    QImage  bodyParts = fileCache.getImage("gfx/interface/Items6.png");

    static const CVec2    partsPos[CCharacter::eBodyPartCount] =
    {
        {34,  59}, // eBodyPartHead
        { 6,  66}, // eBodyPartNeck
        {34,  79}, // eBodyPartTorso
        { 6,  86}, // eBodyPartLeftHand
        {62,  86}, // eBodyPartRightHand
        {34,  99}, // eBodyPartLegs
        {34, 119}  // eBodyPartFeets
    };

    for (int i = 0; i < CCharacter::eBodyPartCount; ++i)
    {
        CRect   rect = getBodyPartRect((CCharacter::EBodyParts)i);
        graph2D.drawImageAtlas(image, partsPos[i], bodyParts, rect);
    }
}

//------------------------------------------------------------
int    CInterface::getWeapon(int num)
{
    CCharacter* c = &game.characters[num];
    return c->bodyObjects[CCharacter::eBodyPartRightHand];
}

//------------------------------------------------------------
CCharacter::SStat   CInterface::getStat(int num, CCharacter::EStatsNames statName)
{
    CCharacter* c = &game.characters[num];
    return c->stats[statName];
}

//------------------------------------------------------------
CRect   CInterface::getChampionPortraitRect(int num)
{
    int x = num % 8;
    int y = num / 8;
    CRect   rect;

    rect.tl.x = x * CHAMP_PORTRAIT_WIDTH;
    rect.tl.y = y * CHAMP_PORTRAIT_HEIGHT;
    rect.br.x = rect.tl.x + CHAMP_PORTRAIT_WIDTH - 1;
    rect.br.y = rect.tl.y + CHAMP_PORTRAIT_HEIGHT - 1;

    return rect;
}

//------------------------------------------------------------
CRect   CInterface::getItemRect(int num)
{
    int x = num % 16;
    int y = num / 16;
    CRect   rect;

    rect.tl.x = x * ITEM_WIDTH;
    rect.tl.y = y * ITEM_HEIGHT;
    rect.br.x = rect.tl.x + ITEM_WIDTH - 1;
    rect.br.y = rect.tl.y + ITEM_HEIGHT - 1;

    return rect;
}

//------------------------------------------------------------
CRect   CInterface::getBodyPartRect(CCharacter::EBodyParts part)
{
    static const int    parts[CCharacter::eBodyPartCount] =
    {
        24, // eBodyPartHead
        16, // eBodyPartNeck
        26, // eBodyPartTorso
        20, // eBodyPartLeftHand
        22, // eBodyPartRightHand
        28, // eBodyPartLegs
        30  // eBodyPartFeets
    };

    return getItemRect(parts[part]);
}

//------------------------------------------------------------
void   CInterface::drawText(QImage* image, CVec2 pos, EFonts font, const char* text, QColor color)
{
    // Sprite number tables for each character in ASCII from char 32 to char 120
    // The lowercase letters are used for the spells
    //  !"#$%&'()*+,-./
    // 0123456789:;<=>?
    // @ABCDEFGHIJKLMNO
    // PQRSTUVWXYZ[\]^_
    // `abcdefghijklmno
    // pqrstuvwx

    // eFontStandard
    static const int codesStandard[] =
    {
        //      !     "     #     $     %     &     '     (     )     *     +     ,     -     .     /
        0x00, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
        //0     1     2     3     4     5     6     7     8     9     :     ;     <     =     >     ?
        0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
        //@     A     B     C     D     E     F     G     H     I     J     K     L     M     N     O
        0x00, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 0x4C, 0x4D, 0x4E, 0x4F,
        //P     Q     R     S     T     U     V     W     X     Y     Z     [     \     ]     ^     _
        0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F,
        //`     a     b     c     d     e     f     g     h     i     j     k     l     m     n     o
        0x00, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E,
        //p     q     r     s     t     u     v     w     x
        0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77
    };

    // eFontScroll
    static const int codesScroll[] =
    {
        //      !     "     #     $     %     &     '     (     )     *     +     ,     -     .     /
        0x00, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x1D, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F,
        //0     1     2     3     4     5     6     7     8     9     :     ;     <     =     >     ?
        0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F,
        //@     A     B     C     D     E     F     G     H     I     J     K     L     M     N     O
        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
        //P     Q     R     S     T     U     V     W     X     Y     Z     [     \     ]     ^     _
        0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x5B, 0x5C, 0x5D, 0x5E, 0x5F,
        //`     a     b     c     d     e     f     g     h     i     j     k     l     m     n     o
        0x00, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E,
        //p     q     r     s     t     u     v     w     x
        0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77
    };

    // eFontWalls
    static const int codesWalls[] =
    {
        //      !     "     #     $     %     &     '     (     )     *     +     ,     -     .     /
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x21, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1B, 0x1A,
        //0     1     2     3     4     5     6     7     8     9     :     ;     <     =     >     ?
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A,
        //@     A     B     C     D     E     F     G     H     I     J     K     L     M     N     O
        0x1A, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E,
        //P     Q     R     S     T     U     V     W     X     Y     Z     [     \     ]     ^     _
        0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A,
        //`     a     b     c     d     e     f     g     h     i     j     k     l     m     n     o
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A,
        //p     q     r     s     t     u     v     w     x
        0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A, 0x1A
    };

    const int* codes;
    int offset;
    QImage  file;

    if (font == eFontStandard)
    {
        codes = codesStandard;
        file = fileCache.getImage("gfx/interface/Font.png");
        offset = -2;
        pos.x -= 3;
    }
    else if (font == eFontScroll)
    {
        codes = codesScroll;
        file = fileCache.getImage("gfx/interface/Font.png");
        offset = -2;
        pos.x -= 3;
    }
    else
    {
        codes = codesWalls;
        file = fileCache.getImage("gfx/3DView/WallFont.png");
        offset = 0;
    }

    int i = 0;

    while (text[i] != 0)
    {
        int c = codes[text[i] - 32];

        CRect   rect(CVec2(c * 8, 0),
                     CVec2(c * 8 + 7, file.height() - 1));
        CVec2   cPos(pos.x + i * (8 + offset), pos.y);

        graph2D.drawImageAtlas(image, cPos, file, rect, (font != eFontWalls ? color : QColor(0, 0, 0, 0)));
        i++;
    }
}

//------------------------------------------------------------
void   CInterface::update(SMouseArea* clickedArea)
{
    if (clickedArea != NULL)
    {
        if (mouse.mButtonPressing == true)
        {
            switch (clickedArea->type)
            {
                case eMouseArea_Inventory:
                    mainState = eMainInventory;
                    currentChampion = (int)clickedArea->param1;
                break;

                case eMouseArea_3DView:
                    mainState = eMainGame;
                break;

                default:
                break;
            }
        }
    }
}
